package Abenteuer.Daten.Objekte;

import Abenteuer.Daten.Orte.Raum;
import java.util.Vector;
import java.util.Enumeration;
import java.text.NumberFormat;

/**
 * Diese Klasse reprsentiert eine Person. Sie hat einen Namen eine kurze und eine
 * ausfhrliche Beschreibung und besitzt eine bestimmte Tragkraft.
 * 
 * @author Dipl.-Ing. Michael Sonntag
 * @version 1.0
*/
public class Person
{
    /** Statische Liste zur Speicherung aller Personen. 
    Package access, soda auch unser Enumerator darauf
    zugreifen kann
    */
    static Vector personenListe=new Vector();
	
	private String beschreibung;
	private String kurzBeschreibung;
	private String name;
	private double tragkraft=1.0;
	private double derzeitigesGewicht=0.0;	
	private Raum derzeitigerOrt;
	/* Kein externer Zugriff, damit Gewicht funktioniert! */
	private Vector inventar=new Vector();
	
	/**
	"Erzeugt" eine neue Person mit einem Namen und einem Startort
	*/
	public Person(String name,String kurzBeschreibung,String beschreibung,Raum derzeitigerOrt)
	{
        if(name==null || kurzBeschreibung==null || beschreibung==null)
            throw new IllegalArgumentException("Name, Beschreibung und Kurzbeschreibung drfen nicht null sein");
		this.name=name;
		this.derzeitigerOrt=derzeitigerOrt;
		this.kurzBeschreibung=kurzBeschreibung;
		this.beschreibung=beschreibung;
		personenListe.addElement(this);
    }
	
	/**
	Gibt den Raum zurck, in dem Sich die Person befindet
	@return Der Raum, in dem sich die Person zur Zeit befindet
	*/
    public Raum getDerzeitigerOrt()
    {
        return derzeitigerOrt;
    }
    
    /**
    Gibt den Namen der Person zurck
    @return Name der Person
    */
    public String getName()
    {
        return name;
    }
    
    /**
    Bewegt die Person von dem derzeitigen Raum in eine bestimmte Richtung. Gibt es keinen Ausgang
    in diese Richtung, so wird eine Meldung ausgegeben.<br>
    Nur wenn die Person diesen Raum in diese Richtung verlassen darf und den neuen Raum vom derzeitige 
    aus betreten darf, erfolgt tatschlich ein Ortswechsel.<br>
    Anschlieend wird die Beschreibung des neuen (oder alten) Raums ausgegeben.
    */
    public void bewege(int richtung)
    {
        if(derzeitigerOrt==null)
        	return;
        Raum neu=derzeitigerOrt.Ausgang(richtung);
        if(neu!=null)
        {
	       	if(derzeitigerOrt.verlassen(this,richtung) && neu.betreten(this,derzeitigerOrt))
	            derzeitigerOrt=neu;
			derzeitigerOrt.beschreibeRaum(this);	            
        }
        else
            System.out.println("In diese Richtung fuehrt kein Weg!");
    }

	/**
	Fgt dem Inventar einen Gegenstand hinzu. Dies erfolgt nur, wenn zusammen mit dem 
	neuen Gegenstand die maximale Tragkraft nicht berschritten wird.
	@param g Der Gegenstand, den die Person ins Inventar aufnehmen soll
	*/
	public void addGegenstand(Gegenstand g)
	{
		if(!inventar.contains(g))
		{
			if(derzeitigesGewicht+g.getGewicht()>tragkraft)
			{
				System.out.println("Soviel kann ich nicht tragen!");
				return;
			}
			derzeitigesGewicht+=g.getGewicht();
			inventar.addElement(g);
		}
	}
	
	/**
	Entfernt einen Gegenstand aus dem Inventar. Ist er nicht darin enthalten, passiert nichts.	
	*/
	public void removeGegenstand(Gegenstand g)
	{
		if(inventar.contains(g))
		{
			derzeitigesGewicht-=g.getGewicht();
			inventar.removeElement(g);
		}
	}
	
	/**
	Liefert eine Enumeration aller Gegenstnde die diese Person bei sich trgt zurck
	@return Enumeration aller Gegenstnde
	@see java.util.Enumeration
	*/
	public Enumeration gegenstaende()
	{
		return inventar.elements();
	}
	
	/**
	Stellt fest, ob die Person einen Gegenstand mit einem bestimmten Namen bei sich trgt.
	@return Irgendeiner der Gegenstnd mit diesem Namen im Inventar oder null falls keiner gefunden wurde
	*/
	public Gegenstand besitzt(String name)
	{
		Enumeration e=inventar.elements();
		Gegenstand res=null;
		while(e.hasMoreElements() && res==null)
		{
			Gegenstand g=(Gegenstand)e.nextElement();
			if(g.getName().equalsIgnoreCase(name))
				res=g;
		}
		return res;
	}

	/**
	Stellt fest, ob die Person den Gegenstand g besitzt
	@param g Der Gegenstand, dessen Besitzverhtlnis geprft werden soll
	@return True, falls die Person den Gegenstand im Inventar hat
	*/
	public boolean besitztGegenstand(Gegenstand g)
	{
		return inventar.contains(g);
	}

	/**
	Gibt das vollstndige Inventar aus
	*/
	public void druckeInventory()
	{
		System.out.println(name+" besitzt zur Zeit:");
		if(inventar.isEmpty())
		{
			System.out.println("Leider gar nichts!");
			return;
		}
		for(Enumeration e=inventar.elements();e.hasMoreElements();)
		{
			((Gegenstand)e.nextElement()).kurzBeschreibung(); 
     	}
     	NumberFormat nf=NumberFormat.getInstance();
     	nf.setMaximumFractionDigits(2);
     	System.out.println("Gesamtgewicht: "+nf.format(derzeitigesGewicht)+" kg");
	}

	/**
	Gibt eine kurze Beschreibung der Person aus (z. b. bei Erwhnung im Raum).
	*/
	public void kurzBeschreibung()
	{
		System.out.println(kurzBeschreibung);
	}
	
	/**
	Gibt eine ausfhrliche Beschreibung der Person aus (z. b. beim Besichtigen der Person).
	*/
	public void beschreibung()
	{
		System.out.println(beschreibung);
	}
	
	/**
	Liefert eine Aufzhlung aller Personen in einem Raum
	@param raum Der Raum, fr den alle darin befindlichen Personen geliefert werden sollen
	@return Aufzhlung der Personen im Raum
	@see java.util.Enumeration
	*/
	public static Enumeration personenInRaum(Raum raum)
	{
	    return new PersonenEnumeration(raum);
	}
}

/**
Hilfsklasse die alle Personen in einem bestimmten Raum liefert
*/
class PersonenEnumeration implements Enumeration
{
    private Raum raum;
    private Enumeration enum=null;
    private Person naechste=null;
 
    /**
    @param raum Der Raum, fr den alle darin befindlichen Personen geliefert werden sollen
    */
    public PersonenEnumeration(Raum raum)
    {
        this.raum=raum;
        enum=Person.personenListe.elements();
        geheZuNaechsterPerson();
    }
    
    public boolean hasMoreElements()
    {
        return naechste!=null;
    }
    
    public Object nextElement()
    {
        Person res=naechste;
        geheZuNaechsterPerson();
        return res;
    }
    
    private void geheZuNaechsterPerson()
    {
        do
        {
            if(enum.hasMoreElements())
                naechste=(Person)enum.nextElement();
            else
                naechste=null;
        }
        while(naechste!=null && naechste.getDerzeitigerOrt()!=raum);        
    }
}