package Abenteuer.Daten.Orte;

import Abenteuer.Daten.Objekte.Gegenstand;
import Abenteuer.Daten.Objekte.Person;
import Abenteuer.Programm.Adventure;
import java.util.Vector;
import java.util.Enumeration;

/**
 * Die Klasse Raum beschreibt einen einzelnen Raum nach folgenden Merkmalen:
 * <ul>
 * <li>Ein Raum hat eine eindeutige Nummer, die automatisch vergeben wird</li>
 * <li>Ein Raum hat einen Namen</li>
 * <li>Ein Raum hat eine Beschreibung</li>
 * <li>Der Raum hat eine Liste der darin befindlichen sofort sichtbaren Gegenstnde</li>
 * <li>Der Raum hat eine Menge von Ausgngen: die 8 Komparichtungen, hinauf, hinunter, hinein oder hinaus</li>
 * <li>Der Raum ist entweder beleuchtet oder nicht</li>
 * </ul>
 * Diese Klasse verwaltet auch automatisch eine Liste aller Rume im Spiel, um das Speichern
 * zu erleichtern.
 * 
 * @author Dipl.-Ing. Michael Sonntag
 * @version 1.0
 */
public class Raum
{
    /**
    Erzeugt einen neuen Raum mit einem Namen und einer Beschreibung. Er ist beleuchtet, enthlt keine Gegenstnde und hat keinen Ausgang.
    @param raumName Name des Raumes
    @param beschreibung Beschreibung des Raumes
    */
    public Raum(String raumName,String beschreibung)
    {
    }
    
    /**
    Erzeugt einen neuen Raum mit einem Namen und einer Beschreibung. Er ist beleuchtet und enthlt keine Gegenstnde.
    Ausgnge nach N, NO, O, SO und hinauf knnen bergeben werden, es ist daher ein einfacheres Erzeugen von Labyrinthen mglich, wenn von
    der oberen nordstlichen Ecke in Richtung der untere sdwestlichen gearbeitet wird. Die Verbindungen werden 
    auch in der Gegenrichtung gesetzt.
    @param raumName Name des Raumes
    @param beschreibung Beschreibung des Raumes
    @param nachN Raum der erreicht wird, wenn das Kommando "Gehe N" gegeben wird
    @param nachNO Raum der erreicht wird, wenn das Kommando "Gehe NO" gegeben wird
    @param nachO Raum der erreicht wird, wenn das Kommando "Gehe O" gegeben wird
    @param nachSO Raum der erreicht wird, wenn das Kommando "Gehe SO" gegeben wird
    @param nachOben Raum der erreicht wird, wenn das Kommando "Gehe hinauf" gegeben wird
    */
    public Raum(String raumName,String beschreibung,Raum nachN,Raum nachNO,Raum nachO,Raum nachSO,Raum nachOben)
    {
    }
    
    /**
    Hier wird gesetzt, wo es von diesem Raum aus hinein geht, und wohin hinaus.
    Jeder Wert kann null sein, um anzuzeigen, da es hier keinen Weg gibt.
    @param hinein Der Raum der beim Kommando "Hinein" von hier aus erreicht wird
    @param hinaus Der Raum der beim Kommando "Hinaus" von hier aus erreicht wird
    */
    public void setzeEinAus(Raum hinein,Raum hinaus)
    {
    }

    /**
    Hier wird gesetzt, wo es von diesem Raum aus hinauf geht, und wohin hinunter.
    Jeder Wert kann null sein, um anzuzeigen, da es hier keinen Weg gibt.
    @param hinauf Der Raum der beim Kommando "Hinauf" von hier aus erreicht wird
    @param hinunter Der Raum der beim Kommando "Hinunter" von hier aus erreicht wird
    */
    public void setzeRaufRunter(Raum hinauf,Raum hinunter)
    {
    }
    
    /**
    Hier kann ein beliebiger Ausgang gesetzt werden.
    Achtung: der Rckweg wird hier nicht gesetzt, sondern ist
    durch einen eigenen Aufruf zu installieren!
    @param richtung Die Richtung, in der der andere Raum von diesem aus gesehen zu erreichen ist
    @param naechster der Raum, der von hier aus zu erreiche ist
    */
    public void setzeAusgang(int richtung, Raum naechster)
    {
    }
    
    /**
    Wenn der Raum nicht beleuchtet ist und weder die Person eine Lichtquelle besitzt
    noch sich eine im Raum befindet, noch eine andere im Raum befindliche Person eine
    solche besitzt oder sie nicht eingeschaltet ist, ist der Raum finster*/
    protected boolean istBeleuchtet()
    {
    }
    
    /**
    Diese Methode gibt die Beschreibung eines Raumes fr eine bestimmte Person aus.
    Ist der Raum finster, so wird keine Beschreibung ausgegeben.
    Die mglichen Ausgangsrichtungen werden nur ausgegeben, wenn zumindest eine klar sichtbar ist.
    Befindet sich zumindest ein Gegenstand direkt im Sichtfeld, so wird auch eine Liste der
    Gegenstnde ausgegeben.
    Ebenso wird eine Liste der Personen (auer der, fuer die der Raum beschrieben wird) ausgegeben, wenn sich mindestens eine andere im selben Raum aufhlt.
    @param fuerWen Die Person, duer die der Raum beschrieben werden soll
    */
    public void beschreibeRaum(Person fuerWen)
    {
    }
    
    /**
    Gibt die eindeutige Nummer des Raumes zurck
    @return Eindeutige Nummer des Raumes
    */
    public long getRaumNummer()
    {
    }
    
    /**
    Gibt den Namen des Raumes zurck
    @return Name des Raumes
    */
    public String getRaumName()
    {
    }
    
    /**
    Gibt zurck, ob der Raum beleuchtet ist
    @return True wenn beleuchtet
    */
    public boolean getBeleuchtung()
    {
    }
    
    /**
    Setzt die Beleuchtung
    @param beleuchtet Wenn true, kann der Raum auch ohne Lichtquelle jederzeit gesehen werden
    */
    public void setBeleuchtung(boolean beleuchtet)
    {
    }
    
    /**
    Fgt einen Gegenstand zum Raum hinzu
    @param g Der Gegenstand, der hinzugefgt werden soll. Ist er bereits im Raum, erfolgt keine Aktion
    */
    public void addGegenstand(Gegenstand g)
    {
    }
    
    /**
    Entfernt einen Gegenstand aus dem Raum
    @param g Der Gegenstand, der entfernt werden soll. Ist er nicht im Raum, erfolgt keine Aktion
    */
    public void removeGegenstand(Gegenstand g)
    {
    }
    
    /**
    Stellt fest, ob ein Gegenstand mit diesem Namen sich in dem Raum befindet
    @param name Der allgemein bekannte (nicht eindeutige) Name des Gegenstands, der geprft werden soll. 
    @return Den Gegenstand falls enthalten, sonst null
    */
    public Gegenstand enthaeltGegenstand(String name)
    {
    }
    
    /**
    Stellt fest, ob ein Gegenstand sich in dem Raum befindet
    @param g Der Gegenstand, der geprft werden soll. 
    @return True falls enthalten, sonst false
    */
    public boolean enthaeltGegenstand(Gegenstand g)
    {
    }
    
    /**
    Liefert eine Enumeration aller Gegenstnde in diesem Raum zurck
    @return Enumeration aller Gegenstnde
    @see java.util.Enumeration
    */
    public Enumeration gegenstaende()
    {
    }
    
    /**
    Liefert den nchsten Raum zurck, der in eine bestimmte Richtung liegt
    @param richtung Der Ausgang, der benutzt werden soll um den nchsten Raum zu bestimmen.
    @return Den Raum oder null, falls es in diese Richtung keinen Ausgang gibt
    */
    public Raum Ausgang(int richtung)
    {
    }
    
    /**
    Liefert zurck, ob eine Person den Raum in eine bestimmte Richtung hin verlassen darf.
    <b>Hier immer true</b>
    @param wer Wer den Raum verlassen mchte
    @param inRichtung Die Richtung, in die der Raum verlassen werden soll
    @return True falls der Raum verlassen werden darf
    */
    public boolean verlassen(Person wer,int inRichtung)
    {
    }
    
    /**
    Liefert zurck, ob eine Person den Raum von einem bestimmten Raum her betreten darf.
    <b>Liefert hier immer dann false, wenn der Raum unbeleuchtet ist und die Person die Lampe nicht bei sich trgt (Egal ob aktiviert oder nicht)</b>
    @param wer Wer den Raum betreten mchte
    @param vorigerRaum Der Raum, von dem aus dieser betreten werden soll
    @return True falls der Raum betreten werden darf
    */
    public boolean betreten(Person wer,Raum vorigerRaum)
    {
    }
    
    /**
    Liefert ein rechteckiges Array von Rumen zurck, die intern vollstndig verbunden sind. Alle Verbindungen an den Rnder fhren
    zu sich selbst zurck, auer die schrgen, die zum nchsten Raum fhren (NO am nrdlichen Rand fhrt zum nchsten Raum im O).<br>
    Alle Rume sind identisch (selber Namen und selbe Beschreibung).
    @param laenge Ausdehnung in Rumen in O-W Richtung
    @param breite Ausdehnung in Rumen in N-S Richtung
    @return Zweidimensionales rechteckiges Array von Rumen
    */
    public static Raum[][] generiereLabyrinth(int laenge,int breite,String raumName,String beschreibung,boolean beleuchtet)
    {
    }
}
