package Abenteuer.Programm;

import Abenteuer.Daten.Orte.Raum;
import Abenteuer.Daten.Objekte.*;
import Abenteuer.Programm.Parser.Parser;
import Abenteuer.Programm.Kommandos.KommandoObjekt;
import Abenteuer.Programm.Kommandos.EndeKommando;

/**
Hauptprogramm des Textadventures
@author Dipl.-Ing. Michael Sonntag
@version 1.0
*/
public class Adventure
{
    /**
    Der Ausgang des ganzen Labyrinths. Hier gespeichert, damit wir feststellen knnen, ob er erreicht wurde.
    */
	Raum ausgang;
	/**
	Der Held des Abenteuers;
	*/
	Person avatar;
	/**
	Der Parser, der die Eingaben verarbeitet
	*/
	Parser parser=new Parser();

	/**
	Die komplette Taschenlampe. Ist global, um einen einfachen Zugriff zu ermglichen (z. B. Prfung ob es in einem Raum Licht gibt).
	*/
	public static ZustandsGegenstand Lampe;
	
	/*
	Initialisierung der Taschenlampe
	*/
	static
	{
		String[] b={"Die Taschenlampe ist klein und rot. Sie enthaelt Batterien und eine Gluehlampe. Sie ist ausgeschaltet.","Die Taschenlampe ist klein und rot. Sie enthaelt Batterien und eine Gluehlampe. Sie ist eingeschaltet."};
		String[] kb={"Eine kleine Taschenlampe (ausgeschaltet)","Eine kleine Taschenlampe (eingeschaltet)"};
		String[] aend={"Sie schalten die Taschenlampe aus","Sie schalten die Taschenlampe ein"};
		Lampe=new ZustandsGegenstand("Taschenlampe","Taschenlampe_komplett",b,kb,aend,0.11,0);
	}

    /*
    Vorsicht bei Wertnderungen: Zahlen werden als Index in Arrays verwendet!
    */
	/** Norden */
	public static final int N=0;	
	/** Nordosten */
	public static final int NO=1;
	/** Osten */
	public static final int O=2;
	/** Sdosten */
	public static final int SO=3;
	/** Sden */
	public static final int S=4;	
	/** Sdwesten */
	public static final int SW=5;	
	/** Westen */
	public static final int W=6;	
	/** Nordwesten */
	public static final int NW=7;
	/** Hinauf */
	public static final int U=8;	
	/** Hinunter */
	public static final int D=9;	
	/** Hinein */
	public static final int E=10;	
	/** Hinaus */
	public static final int A=11;	
	/** Maximum der Richtungen */
	public static final int MAX_RICHTUNG=11;	
    /**
    Die Namen der Richtungen
    */	
	public static final String[] Richtungen={"Norden","Nordosten","Osten","Suedosten","Sueden","Suedwesten","Westen","Nordwesten","Hinauf","Hinunter","Hinein","Hinaus"}; // Indized mssen mit den Richtungen bereinstimmen

/**
Erzeugt das vollstndige Labyrinth mit Gegenstnden und Personen
*/
public Adventure()
{
	Raum eingang=new Raum("Eingang","Sie stehen vor dem Eingang einer finsteren Hoehle in einem hohen Berg, den sie unbedingt ueber- (oder unter-?)queren muessen. Stellenweise sind schwache Fackeln in rostigen Leuchtern montiert, deren Licht ab und zu ein Ratte bei ihrer Nahrungssuche hilft.");
	Raum hoehle=new Raum("Hoehleneingang","Sie befinden sich am Eingang einer Hoehle, die tief in den Berg zu fuehren scheint");
	eingang.setzeEinAus(hoehle,null);
	new Person("Geist","Ein Geist","Der Geist wandert einfach durch den Raum. Nichts was Sie tun, scheint auch nur den geringsten Einflusz auf ihn zu haben.",hoehle);
	Gegenstand g=new EinfacherGegenstand("Taschenlampe","Taschenlampe_ohne_Birne","Die Taschenlampe ist klein und rot. Als Sie versuchen Sie einzuschalten, gibt sie kein Licht. Bei einer nheren Untersuchung stellen Sie fest, da zwar Batterien enthalten sind, die Gluehbirne aber fehlt!","Eine kleine Taschenlampe (ohne Gluehbirne)",0.1);
	hoehle.addGegenstand(g);
	g=new EinfacherGegenstand("Gluehbirne","Taschenlampenbirne","Eine kleine Gluehbirne fuer eine Taschenlampe","Eine kleine Gluehbirne",0.01);
	hoehle.addGegenstand(g);
	g=new EinfacherGegenstand("Bananenschale","Bananenschale","Eine Bananenschale auf der man leicht ausrutschen kann.","Eine Bananenschale",0.05);
	hoehle.addGegenstand(g);
	g=new EinfacherGegenstand("Goldmuenze","Philharmoniker","Eine Goldmuenze (genauer: ein goldener Philharmoniker)!","Eine Goldmuenze",0.1);
	hoehle.addGegenstand(g);
	g=new EinfacherGegenstand("Nagel","Nagel_verbogen","Wie kommt blosz dieser rostige und verbogene alte Nagel in diese Hoehle?!?","Ein rostiger Nagel (verbogen)",0.01);
	hoehle.addGegenstand(g);
	g=new EinfacherGegenstand("Nagel","Nagel_ausgebogen","Wie kommt blosz dieser rostige alte Nagel in diese Hoehle?!?","Ein rostiger Nagel",0.01);
	// Der ausgebogene Nagel wird nur erzeugt. Erst beim benutzen des verbogenen wird dieser gegen den ausgebogenen ausgetauscht
	Raum[][] laby=Raum.generiereLabyrinth(4,3,"Finstere Hoehle","Sie befinden sich in einer finsteren Hoehle. Ab und zu hoeren sie knapp auszerhalb des Lichts etwas Rascheln. Ob das Ratten sind?",false);
	hoehle.setzeAusgang(Adventure.O,laby[1][0]);
	laby[1][0].setzeAusgang(Adventure.W,hoehle);
	// Der Ausgang der Labyrinths
	ausgang=new Raum("Hoehlenausgang","Sie haben es geschafft! Sie haben die Hoehle durchquert und die andere Seite des Berges erreicht!");
	ausgang.setzeAusgang(Adventure.NW,laby[2][3]);
	laby[2][3].setzeAusgang(Adventure.SO,ausgang);
	new Person("Bettler","Ein Bettler","Der Bettler ist anscheinend stumm, da er nicht antwortet als Sie ihn ansprechen, sondern er zeigt lediglich nach Suedosten.",laby[2][3]);
	// Der Held
	avatar=new Person("Ich","Der Held des Abenteuers","Ein groszartiger und mutiger Abenteurer",eingang);
}

/**
Hauptschleife des Abenteuers
@return True wenn der Ausgang erreicht wurde, false bei Tod oder vorzeitiger Beendigung
*/
private boolean spielen()
{
    // Den Start beschreiben
	avatar.getDerzeitigerOrt().beschreibeRaum(avatar);
	KommandoObjekt kommando;
	// Hauptschleife: So lange nicht beendet oder Erfolg, Kommando einlesen und ausfhren
	while(avatar.getDerzeitigerOrt()!=ausgang && !((kommando=parser.getNaechstesKommando(avatar)) instanceof EndeKommando))
		kommando.ausfuehren();
	//
	return avatar.getDerzeitigerOrt()==ausgang;
}

/**
Einsprungpunkt
*/
static public void main(String args[])
{
    System.out.println("Willkommen beim groszen Hoehlen-Abenteuer!\nUm eine Befehlsuebersicht zu erhalten, geben Sie \"hilfe\" ein.\n\n");
	if(new Adventure().spielen())
		System.out.println("Gratuliere!");
}

}

